package id

import "fmt"

const (
	Service        = "service"
	ServicesPrefix = "services"
)

type ServiceId struct {
	ServiceID string `json:"service_id,omitempty" bson:"service_id,omitempty"`
}

var _ Descriptor = &ServiceId{}

func (id *ServiceId) New() Descriptor {
	return &ServiceId{}
}

func (id *ServiceId) Type() string { return Service }

func (id *ServiceId) String() string {
	return Join(ServicesPrefix, id.ServiceID)
}

func (id *ServiceId) FromParts(parts []string) error {
	if len(parts) != 2 || parts[0] != ServicesPrefix {
		return ErrInvalidID
	}
	id.ServiceID = parts[1]
	return nil
}

func (id *ServiceId) Map() map[string]any {
	return map[string]any{
		"service_id": id.ServiceID,
		"type":       id.Type(),
	}
}

func (id *ServiceId) FromMap(m map[string]any) error {
	id.ServiceID, _ = m["service_id"].(string)
	if id.ServiceID == "" {
		return fmt.Errorf("%w: ServiceId required", ErrInvalidID)
	}
	return nil
}

func (id *ServiceId) Validate() error {
	if id.ServiceID == "" {
		return fmt.Errorf("%w: ServiceId required", ErrInvalidID)
	}
	return nil
}

func NewServiceID(id string) *ObjectId {
	return &ObjectId{Descriptor: &ServiceId{ServiceID: id}}
}
