package expr

import (
	exprcompiler "github.com/expr-lang/expr/compiler"
	"github.com/expr-lang/expr/parser"
	"github.com/expr-lang/expr/vm"
	"golang.org/x/net/context"
)

const EnvContextKey = "$context"

func Eval(ctx context.Context, input string, env map[string]interface{}) (interface{}, error) {
	tree, err := parser.Parse(input)
	if err != nil {
		return nil, err
	}

	e := GetEnv(ctx)

	if e == nil {
		e = make(map[string]interface{})
	}

	for k, v := range env {
		e[k] = v
	}

	e[EnvContextKey] = ctx
	cfg := GetDefaultConfig(e)

	program, err := exprcompiler.Compile(tree, cfg)
	if err != nil {
		return nil, err
	}

	output, err := vm.Run(program, e)
	if err != nil {
		return nil, err
	}

	return output, nil
}

func EvalKV(ctx context.Context, input string, kv ...interface{}) (interface{}, error) {
	m := make(map[string]interface{})

	for i := 0; i < len(kv)/2; i++ {
		key, ok := kv[i].(string)
		if !ok {
			panic("key should be string")
		}
		m[key] = kv[i+1]
	}

	return Eval(ctx, input, m)
}
